'' =================================================================================================
''
''   File....... jm_etimer.spin (version 1.2)
''   Purpose.... Background timer
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009-2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 21 JUL 2009
''   Updated.... 15 JUN 2013
''
'' =================================================================================================

{{

  Version 1.2  * Updated for par-only interface to cog
               * fixed bug in hold() method
               * renamed register constants
               * readreg and writereg renamed read_reg and write_reg

  Version 1.1  * Reveals address of registers through address() method
               * Days from 0 to 6 (Sun to Sat)
               * clear() method renamed reset() 

}}


con

  #0, REG_MS, REG_SC, REG_MN, REG_HR, REG_DY

  #0, ET_HOLD, ET_RUN
  

var

  long  cog                                                     ' cog (+1) running timer

  long  hbpin                                                   ' hearbeat pin (if used)
  long  mstix                                                   ' system ticks in 1ms
  long  timermode                                               ' 0 = hold, else run
  long  timer[5]                                                ' event timer


pub start(p, mode) 

'' Starts event timer driver
'' -- p is "heartbeat" pin (-1 if not used)
'' -- mode is run mode (0 to stop, non-zero to run)

  stop                                                          ' stop if already running 

  hbpin := p                                                    ' copy pin
  mstix := clkfreq / 1_000                                      ' calcuate tix per millisecond

  timermode := ET_HOLD                                          ' set to hold
  
  cog := cognew(@etimer, @hbpin) + 1                            ' start timer cog

  if (cog)                                                      ' did cog launch?
    reset                                                       '  clear the registers
    timermode := mode                                           '  set mode as requested
    
  return cog


pub stop

'' Stops event timer; frees a cog 

  if (cog)
    cogstop(cog - 1)
    reset   


pub read(p_dest)

'' Copies timer registers to long array at p_dest

  longmove(p_dest, @timer, 5)  


pub write(p_src)

'' Copies values in long array at p_src to timer registers

  longmove(@timer, p_src, 5)


pub set(ms, sc, mn, hr, dy)

'' Set individual clock registers

  longmove(@timer, @ms, 5)
  
  
pub read_reg(reg) 

'' Reads selected register from the timer

  if ((reg => REG_MS) and (reg =< REG_DY))
    return etimer[reg]
  else
    return -1

    
pub write_reg(reg, value)

'' Returns selected register from the timer

  case reg
    REG_MS:
      if (value => 0) and (value =< 999)
        etimer[REG_MS] := value
        return true
      else
        return false        

    REG_SC:
      if (value => 0) and (value =< 59)
        etimer[REG_SC] := value
        return true
      else
        return false    

    REG_MN:
      if (value => 0) and (value =< 59)
        etimer[REG_MN] := value
        return true
      else
        return false 

    REG_HR:
      if (value => 0) and (value =< 23)
        etimer[REG_HR] := value
        return true
      else
        return false 

    REG_DY:
      if (value => 0) and (value =< 6)
        etimer[REG_DY] := value
        return true
      else
        return false 

    other:
      return false
      
  
pub hold

'' Stops timer; does not clear

  timermode := ET_HOLD


pub resume

'' Restarts timer; does not clear 

  timermode := ET_RUN


pub running

'' Returns running status as true or false

  return (timermode <> ET_HOLD)
    

pub reset

'' Clears timer value; does not stop timer

  longfill(@timer, 0, 5)


pub address

'' Returns address of timer regs

  return @timer


dat

                        org     0

etimer                  mov     t1, par                         ' start of parameters
                        rdlong  t2, t1                          ' read hb pin
                        cmps    t2, #0                  wc      ' check for less than 0
        if_b            mov     hbmask, #0                      ' clear heartbeat pin mask
        if_ae           mov     hbmask, #1                      ' create pin mask
        if_ae           shl     hbmask, t2
        if_ae           andn    outa, hbmask                    ' start off
        if_ae           or      dira, hbmask                    ' set to output mode

                        add     t1, #4                          ' point to mstix
                        rdlong  ms001, t1

                        add     t1, #4
                        mov     mdpntr, t1                      ' save address of mode

                        add     t1, #4
                        mov     mspntr, t1                      ' save address of milliseconds

                        add     t1, #4
                        mov     scpntr, t1                      ' save address of seconds

                        add     t1, #4
                        mov     mnpntr, t1                      ' save address of minutes

                        add     t1, #4
                        mov     hrpntr, t1                      ' save address of hours

                        add     t1, #4
                        mov     dypntr, t1                      ' save address of days


starttimer              mov     ms1timer, ms001                 ' initialize ms timer
                        add     ms1timer, cnt                   ' sync with system timer

checkmode               rdlong  t1, mdpntr              wz      ' read timer mode
                        or      hbmask, hbmask          wc      ' mask used?
        if_z_and_c      andn    outa, hbmask                    ' kill hb led
        if_z            jmp     #starttimer                     ' if hold, try again

                        waitcnt ms1timer, ms001                 ' run the timer
  
getms                   rdlong  t1, mspntr                      ' read current ms

heartbeat               tjz     hbmask, #updatems               ' skip if no hb
                        cmp     t1, #0                  wz      ' if 0
        if_e            xor     outa, hbmask                    '   toggle
        if_e            jmp     #updatems
                        cmp     t1, #500                wz      ' or 500
        if_e            xor     outa, hbmask                    '   toggle

updatems                add     t1, #1                          ' increment
                        cmp     t1, SEC_01              wz      ' at limit?
        if_e            mov     t1, #0                          ' if yes, clear
                        wrlong  t1, mspntr                      ' write back new value
        if_ne           jmp     #checkmode                      ' to top if no roll-over
                
updatesecs              rdlong  t1, scpntr
                        add     t1, #1
                        cmp     t1, #60                 wz
        if_e            mov     t1, #0
                        wrlong  t1, scpntr
        if_ne           jmp     #checkmode                            

updatemins              rdlong  t1, mnpntr
                        add     t1, #1
                        cmp     t1, #60                 wz
        if_e            mov     t1, #0
                        wrlong  t1, mnpntr
        if_ne           jmp     #checkmode

updatehrs               rdlong  t1, hrpntr
                        add     t1, #1
                        cmp     t1, #24                 wz
        if_e            mov     t1, #0
                        wrlong  t1, hrpntr
        if_ne           jmp     #checkmode

updatedys               rdlong  t1, dypntr
                        add     t1, #1
                        cmp     t1, #7                  wz
        if_e            mov     t1, #0
                        wrlong  t1, dypntr
                        
                        jmp     #checkmode                      ' done, back to top
                
' --------------------------------------------------------------------------------------------------

SEC_01                  long    1_000                           ' ms in 1s

hbmask                  res     1                               ' heartbeat pin mask
ms001                   res     1                               ' ticks in 1ms

mdpntr                  res     1                               ' hub address of timer mode
mspntr                  res     1                               ' hub address of milliseconds
scpntr                  res     1                               ' hub address of seconds
mnpntr                  res     1                               ' hub address of minutes
hrpntr                  res     1                               ' hub address of hours
dypntr                  res     1                               ' hub address of days

ms1timer                res     1                               ' for 1ms timing

t1                      res     1                               ' work vars
t2                      res     1

                        fit     496
                        

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}